"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "mutateUtil", {
  enumerable: true,
  get: function get() {
    return _immutabilityHelper.default;
  }
});
exports.buildIndex = exports.checkCastById = exports.flattenMutationKeys = exports.filter = exports.pickBy = exports.mingoTester = exports.mingoQuery = exports.genTmpId = exports.getDeviceName = exports.tmpIdRegExp = exports.TMP_ID_PREFIX = void 0;

var _mapValues2 = _interopRequireDefault(require("lodash/mapValues"));

var _groupBy2 = _interopRequireDefault(require("lodash/groupBy"));

var _keyBy2 = _interopRequireDefault(require("lodash/keyBy"));

var _each2 = _interopRequireDefault(require("lodash/each"));

var _transform2 = _interopRequireDefault(require("lodash/transform"));

var _keys2 = _interopRequireDefault(require("lodash/keys"));

var _filter2 = _interopRequireDefault(require("lodash/filter"));

var _values2 = _interopRequireDefault(require("lodash/values"));

var _pickBy2 = _interopRequireDefault(require("lodash/pickBy"));

var _pick2 = _interopRequireDefault(require("lodash/pick"));

var _omitBy2 = _interopRequireDefault(require("lodash/omitBy"));

var _startsWith2 = _interopRequireDefault(require("lodash/startsWith"));

var _mingo = _interopRequireDefault(require("mingo"));

var _immutabilityHelper = _interopRequireDefault(require("immutability-helper"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TMP_ID_PREFIX = 'dv~';
exports.TMP_ID_PREFIX = TMP_ID_PREFIX;
var tmpIdRegExp = /^dv~(\d{4}-[01]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d\.\d+Z)~([.\d]+)~(.+)/;
exports.tmpIdRegExp = tmpIdRegExp;

var getDeviceName = function getDeviceName(db) {
  return db && db.deviceName || 'tmp';
}; // NOTE query key with $$ prefix are omitted in query but send to fetcher


exports.getDeviceName = getDeviceName;

var startsWith$$ = function startsWith$$(v, k) {
  return (0, _startsWith2.default)(k, '$$');
};

var genTmpId = function genTmpId(deviceName) {
  return "".concat(TMP_ID_PREFIX).concat(new Date().toISOString(), "~").concat(Math.random(), "~").concat(deviceName || '');
};

exports.genTmpId = genTmpId;

var mingoQuery = function mingoQuery(query) {
  return new _mingo.default.Query((0, _omitBy2.default)(query, startsWith$$));
};

exports.mingoQuery = mingoQuery;

var mingoTester = function mingoTester(query) {
  var mQuery = mingoQuery(query);
  return function (doc) {
    return doc && mQuery.test(doc);
  };
};

exports.mingoTester = mingoTester;

var pickBy = function pickBy(byId, query) {
  if (typeof query === 'string' || Array.isArray(query)) return (0, _pick2.default)(byId, query);
  if (!query) return byId;
  return (0, _pickBy2.default)(byId, mingoTester(query));
};

exports.pickBy = pickBy;

var filter = function filter(byId, query) {
  if (!query) return (0, _values2.default)(byId);
  return (0, _filter2.default)(byId, mingoTester(query));
};

exports.filter = filter;

_immutabilityHelper.default.extend('$auto', function (value, object) {
  return object ? (0, _immutabilityHelper.default)(object, value) : (0, _immutabilityHelper.default)({}, value);
});

// @auto-fold
var flattenMutationKeys = function flattenMutationKeys(mutation, skipUnset) {
  return (0, _keys2.default)((0, _transform2.default)(mutation, function (ret, value, key) {
    if (key[0] === '$') {
      if (!skipUnset && key === '$unset') {
        (0, _each2.default)(value, function (k) {
          ret[k] = true;
        });
      } else if (key === '$merge' || key === '$toggle') {
        (0, _each2.default)(value, function (v, k) {
          ret[k] = true;
        });
      }
    } else {
      ret[key] = true;
    }
  }));
}; // @auto-fold here prepare cast by loop mutation.byId and mark [id] $toggle $merge


exports.flattenMutationKeys = flattenMutationKeys;

var checkCast = function checkCast(coll, nextById, prevById, id) {
  var nextDoc = nextById[id];

  if (nextDoc !== prevById[id] && nextDoc) {
    nextById[id] = coll.cast(nextDoc) || nextDoc;
  }
}; // @auto-fold


var checkCastById = function checkCastById(space, next, prev, mutation) {
  var nextById = next[space];
  var prevById = prev[space];
  if (nextById === prevById) return;
  flattenMutationKeys(mutation[space], true).forEach(function (id) {
    checkCast(next, nextById, prevById, id);
  });
};

exports.checkCastById = checkCastById;

var buildIndex = function buildIndex(docs, fields, isUnique) {
  fields = Array.isArray(fields) ? fields : [fields];
  var field = fields[0];

  if (fields.length === 1) {
    return isUnique ? (0, _keyBy2.default)(docs, field) : (0, _groupBy2.default)(docs, field);
  }

  var restSteps = fields.slice(1);
  var groups = (0, _groupBy2.default)(docs, field);
  return (0, _mapValues2.default)(groups, function (groupDocs) {
    return buildIndex(groupDocs, restSteps, isUnique);
  });
};

exports.buildIndex = buildIndex;