"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.forkDb = void 0;

var _omit2 = _interopRequireDefault(require("lodash/omit"));

var _pick2 = _interopRequireDefault(require("lodash/pick"));

var _map2 = _interopRequireDefault(require("lodash/map"));

var _mapValues2 = _interopRequireDefault(require("lodash/mapValues"));

var _each2 = _interopRequireDefault(require("lodash/each"));

var _db = _interopRequireDefault(require("./db"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var parentDataMutation = ['preloads', 'fetchAts', 'fetchingAt', '_fetchResults', '_byIdAts'];

var forkDb = function forkDb(parentDb) {
  var confs = parentDb.getConfig();
  var subDb = (0, _db.default)(confs); // copy submits and originals

  (0, _each2.default)(confs, function (conf, name) {
    subDb[name].submits = _objectSpread({}, parentDb[name].submits);
    subDb[name].originals = _objectSpread({}, parentDb[name].originals);
  }); // subscribe parent change to child db change

  parentDb.subscribe(function (parentChange) {
    // re-emit parent change event by ...
    var subChange = (0, _mapValues2.default)(parentChange, function (coll, name) {
      return (// create new instance and clear _cache
        _objectSpread({}, subDb[name], {
          _cache: {}
        })
      );
    });
    Object.assign(subDb, subChange);
    subDb.emit(subChange);
  });
  return Object.assign(subDb, {
    getParent: function getParent() {
      return parentDb;
    },
    getFetchData: function getFetchData(name) {
      return parentDb.getDb()[name];
    },
    dispatchFilter: function dispatchFilter(mutSpecs) {
      var parentMuts = [];
      var newSpecs = (0, _map2.default)(mutSpecs, function (mutSpec) {
        var name = mutSpec.name,
            mutation = mutSpec.mutation; // TODO separate/handle $merge, $toggle, $unset

        if (parentDataMutation.find(function (key) {
          return mutation[key];
        })) {
          parentMuts.push({
            name: name,
            mutation: (0, _pick2.default)(mutation, parentDataMutation)
          });
          return {
            name: name,
            mutation: (0, _omit2.default)(mutation, parentDataMutation)
          };
        }

        return mutSpec;
      });

      if (parentMuts.length > 0) {
        parentDb.dispatch(parentMuts);
      }

      return newSpecs;
    }
  });
};

exports.forkDb = forkDb;