"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _isPlainObject2 = _interopRequireDefault(require("lodash/isPlainObject"));

var _pickBy2 = _interopRequireDefault(require("lodash/pickBy"));

var _mapValues2 = _interopRequireDefault(require("lodash/mapValues"));

var _each2 = _interopRequireDefault(require("lodash/each"));

var _collectionCore = _interopRequireDefault(require("./collectionCore"));

var _collectionRead = _interopRequireDefault(require("./collectionRead"));

var _collectionWrite = _interopRequireDefault(require("./collectionWrite"));

var _collectionUtil = require("./collection-util");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var createDb = function createDb(config) {
  var subscribers = [];

  var subscribe = function subscribe(subscriber) {
    subscribers.push(subscriber);
    var isSubscribed = true;
    return function unsubscribe() {
      if (!isSubscribed) return false;
      isSubscribed = false;
      subscribers.splice(subscribers.indexOf(subscriber), 1);
      return true;
    };
  };

  var canEmit = false;

  var db = _objectSpread({}, _collectionCore.default, _collectionRead.default, _collectionWrite.default, {
    dispatch: function dispatch(actionsOrName, mut) {
      // normalize
      var actions = Array.isArray(actionsOrName) ? actionsOrName : [{
        name: actionsOrName,
        mutation: mut
      }]; // hook

      canEmit = false;
      if (db.dispatchFilter) actions = db.dispatchFilter(actions); // do change

      var change = {};
      var hasChange = false;
      (0, _each2.default)(actions, function (_ref) {
        var name = _ref.name,
            mutation = _ref.mutation;
        var prev = db[name];
        var next = (0, _collectionUtil.mutateUtil)(prev, mutation);

        if (next !== prev) {
          next._cache = {};

          if (next.cast) {
            (0, _collectionUtil.checkCastById)('submits', next, prev, mutation);
            (0, _collectionUtil.checkCastById)('preloads', next, prev, mutation);
          }

          db[name] = change[name] = next;
          hasChange = true;
        }
      });
      canEmit = true; // assign change to db and fire to subscribers

      if (hasChange) db.emit(change);
    },
    subscribe: subscribe,
    emit: function emit(change) {
      if (canEmit) (0, _each2.default)(subscribers, function (subscriber) {
        return subscriber(change);
      });
    },
    getConfig: function getConfig() {
      return config;
    },
    getDb: function getDb() {
      return db;
    } // create collections

  });

  var colls = (0, _mapValues2.default)((0, _pickBy2.default)(config, _isPlainObject2.default), function (conf, name) {
    return _objectSpread({
      // local persist
      submits: {},
      originals: {},
      // local memory
      _cache: {},
      // fetch persist
      preloads: {},
      fetchAts: {},
      // fetch memory
      fetchingAt: null,
      _fetchResults: {},
      _fetchPromises: {},
      _byIdAts: {},
      TYPE: 'Collection',
      idField: '_id',
      name: name,
      getDb: function getDb() {
        return db;
      }
    }, conf);
  });
  Object.assign(db, colls); // init collections

  (0, _each2.default)(colls, function (coll, name) {
    if (coll.initState) db.load(name, coll.initState);
    coll = db[name];
    if (coll.onInit) coll.onInit(coll);
  });
  canEmit = true;
  return db;
};

var _default = createDb;
exports.default = _default;