import _values from "lodash/values";

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import stringify from 'fast-stable-stringify';
import { genTmpId, getDeviceName, pickBy, buildIndex, flattenMutationKeys } from './collection-util'; // @auto-fold here

var tryCache = function tryCache(cache, key, func) {
  var c = cache[key];
  if (c) return c;
  return cache[key] = func(); // eslint-disable-line
};

var _getLocalData = function _getLocalData(db, name, field, funcName) {
  var coll = db.getLocalData(name);
  var fn = coll[funcName];
  return typeof fn === 'function' ? fn(coll) : coll[field];
};

export default {
  getFetchData: function getFetchData(name) {
    return this.getDb()[name];
  },
  getLocalData: function getLocalData(name) {
    return this.getDb()[name];
  },
  getSubmits: function getSubmits(name) {
    return _getLocalData(this, name, 'submits', 'getSubmits');
  },
  getOriginals: function getOriginals(name) {
    return _getLocalData(this, name, 'originals', 'getOriginals');
  },
  getPreloads: function getPreloads(name) {
    return this.getFetchData(name).preloads;
  },
  getById: function getById(name) {
    var _this = this;

    return tryCache(this.getLocalData(name)._cache, 'byId', function () {
      return _objectSpread({}, _this.getPreloads(name), _this.getSubmits(name));
    });
  },
  pickInMemory: function pickInMemory(name, query) {
    return pickBy(this.getById(name), query);
  },
  findInMemory: function findInMemory(name, query) {
    return _values(this.pickInMemory(name, query));
  },
  recall: function recall(name, fnName) {
    var _this2 = this;

    for (var _len = arguments.length, args = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
      args[_key - 2] = arguments[_key];
    }

    var coll = this.getLocalData(name);
    var func = coll[fnName] || (fnName === 'buildIndex' ? buildIndex : null);
    return tryCache(coll._cache, "".concat(fnName, "-").concat(stringify(args)), function () {
      return func.apply(coll, [_this2.getById(name)].concat(args));
    });
  },
  genId: function genId() {
    return genTmpId(getDeviceName(this));
  },
  getPending: function getPending(name) {
    var promises = Object.values(this[name]._fetchPromises);
    return promises.length <= 0 ? null : Promise.all(promises);
  },
  mutate: function mutate(name) {
    for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
      args[_key2 - 1] = arguments[_key2];
    }

    var mutSubmits = args.reduceRight(function (ret, step) {
      return _defineProperty({}, step, ret);
    });
    var newOriginals = {};
    var oldSubmits = this.getSubmits(name);
    var oldOriginals = this.getOriginals(name);
    var oldPreloads = this.getPreloads(name); // copy preloads to originals

    flattenMutationKeys(mutSubmits).forEach(function (id) {
      if (!(id in oldSubmits)) {
        // copy to submits to prepare mutation
        oldSubmits[id] = oldPreloads[id];
      }

      if (!(id in oldOriginals)) {
        // need to convert undefined original to null, for persist
        var newOriginal = oldPreloads[id];
        newOriginals[id] = newOriginal === undefined ? null : newOriginal;
      }
    });
    this.dispatch(name, {
      submits: mutSubmits,
      originals: {
        $merge: newOriginals
      }
    });
  }
};