import _size from "lodash/size";
import _every from "lodash/every";
import _get from "lodash/get";
import _sortedUniq from "lodash/sortedUniq";
import _filter from "lodash/filter";
import _startsWith from "lodash/startsWith";
import _omitBy from "lodash/omitBy";

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import stringify from 'fast-stable-stringify';
import { TMP_ID_PREFIX } from './collection-util';
export var defaultGetFetchKey = function defaultGetFetchKey(query, option) {
  return stringify(_objectSpread({}, _omitBy(option, function (v, k) {
    return k[0] === '_';
  }), query));
}; // @auto-fold here

var markPromise = function markPromise(coll, fetchKey, func) {
  var _fetchPromises = coll._fetchPromises;
  var oldPromise = _fetchPromises[fetchKey];
  if (oldPromise) return oldPromise;

  var markPromiseDone = function markPromiseDone() {
    delete _fetchPromises[fetchKey];

    if (Object.keys(_fetchPromises).length === 0) {
      coll.getDb().dispatch(coll.name, {
        fetchingAt: {
          $set: undefined
        }
      });
    }
  };

  var promise = _fetchPromises[fetchKey] = func().then(function (ret) {
    markPromiseDone();
    return ret;
  }).catch(function (err) {
    markPromiseDone();
    return Promise.reject(err);
  }); // ensure fetchingAt is set to coll instantaneously

  var fetchingAt = Date.now();
  coll.getDb().dispatch(coll.name, {
    fetchingAt: {
      $set: fetchingAt
    }
  });
  return promise;
};

var normalizeQueryBasic = function normalizeQueryBasic(query, idField) {
  // query = [ids]
  if (Array.isArray(query)) {
    if (query.length === 0) return false;
    return _defineProperty({}, idField, {
      $in: query
    });
  }

  var fetchQuery = _objectSpread({}, query);

  if (idField in fetchQuery) {
    var idFieldValue = fetchQuery[idField];

    if (!idFieldValue) {
      // query = { id: falsy }
      return false;
    } else if (typeof idFieldValue === 'string') {
      // query = { id: idStr }
      fetchQuery[idField] = {
        $in: [idFieldValue]
      };
    }
  }

  return fetchQuery;
};

var isTmpId = function isTmpId(id) {
  return !id || _startsWith(id, TMP_ID_PREFIX);
};

var sortUniqFilter = function sortUniqFilter(ids) {
  return _filter(_sortedUniq(ids.sort()), function (id) {
    return !isTmpId(id);
  });
}; // @auto-fold here


var defaultGetFetchQuery = function defaultGetFetchQuery(fetchQuery) {
  var entries = Object.entries(fetchQuery);

  for (var i = 0, ii = entries.length; i < ii; i++) {
    var _entries$i = _slicedToArray(entries[i], 2),
        key = _entries$i[0],
        matcher = _entries$i[1];

    if (matcher) {
      if (typeof matcher === 'string' && isTmpId(matcher)) {
        return false;
      } else if (matcher.$in) {
        var $in = sortUniqFilter(matcher.$in);

        if ($in.length === 0) {
          return false;
        }

        fetchQuery[key] = {
          $in: $in
        };
      }
    }
  }

  return fetchQuery;
};

var isAllIdHit = function isAllIdHit(coll, query) {
  var ids = _get(query, [coll.idField, '$in']);

  if (!ids) return false;
  var expire = coll.fetchMaxAge > 0 ? Date.now() - coll.fetchMaxAge : 0;
  return _every(ids, function (id) {
    return coll._byIdAts[id] > expire;
  });
};

function doFetch(db, name, query, option) {
  var coll = db.getFetchData(name);
  if (!coll.onFetch) return undefined; // use getFetchQuery to modify final ajax call query

  var fetchQuery = normalizeQueryBasic(query, coll.idField);
  if (fetchQuery === false) return undefined;
  fetchQuery = (coll.getFetchQuery || defaultGetFetchQuery)(fetchQuery, coll.idField, coll);
  if (fetchQuery === false) return undefined;

  var fetchOption = _omitBy(option, function (v, k) {
    return k[0] === '_';
  });

  if (_size(fetchOption) === 0 && isAllIdHit(coll, fetchQuery)) return undefined; // use getFetchKey to stringify into querystring like key

  var fetchKey = (coll.getFetchKey || defaultGetFetchKey)(fetchQuery, option);
  if (fetchKey === false) return undefined;
  option._fetchKey = fetchKey; // collection.fetchMaxAge: 1, // in seconds; null, 0 or -1 means no maxAge

  var now = Date.now(); // console.log('>>>', fetchKey, fetchAts, fetchAts[fetchKey])

  if (coll.fetchMaxAge > 0 ? coll.fetchAts[fetchKey] > now - coll.fetchMaxAge : coll.fetchAts[fetchKey]) {
    if (option._keepFetchResult) return coll._fetchPromises[fetchKey] || coll._fetchResults[fetchKey];
    return false;
  } // doFetch


  coll.fetchAts[fetchKey] = Date.now();
  return markPromise(coll, fetchKey, function () {
    return Promise.resolve(coll.onFetch(fetchQuery, option, coll)).then(function (res) {
      if (option._keepFetchResult) coll._fetchResults[fetchKey] = res;
      db.load(name, res);
      return res;
    });
  });
}

export default {
  fetch: function fetch(name, query) {
    var option = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    option._keepFetchResult = true;
    return doFetch(this, name, query, option);
  },
  get: function get(name, id) {
    var option = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    doFetch(this, name, [id], option);
    return this.getById(name)[id];
  },
  pick: function pick(name, query) {
    var option = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    doFetch(this, name, query, option);
    return this.pickInMemory(name, query);
  },
  find: function find(name, query) {
    var option = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    doFetch(this, name, query, option);
    return this.findInMemory(name, query);
  },
  pickAsync: function pickAsync(name, query) {
    var _this = this;

    var option = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    return Promise.resolve(doFetch(this, name, query, option)).then(function () {
      return _this.pickInMemory(name, query);
    });
  },
  findAsync: function findAsync(name, query) {
    var _this2 = this;

    var option = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    return Promise.resolve(doFetch(this, name, query, option)).then(function () {
      return _this2.findInMemory(name, query);
    });
  }
};