import _map from "lodash/map";
import _each from "lodash/each";
import _isPlainObject from "lodash/isPlainObject";
import _mapValues from "lodash/mapValues";
import _keys from "lodash/keys";
import _mapKeys from "lodash/mapKeys";
import _flatMap from "lodash/flatMap";
import _compact from "lodash/compact";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

// @auto-fold here
var runCast = function runCast(coll, funcName, doc) {
  if (coll[funcName]) {
    var newDoc = coll[funcName](doc);
    return newDoc === undefined ? doc : newDoc;
  }

  return doc;
};

export default {
  load: function load(name, res, returnMutation) {
    var _this = this;

    if (!name) return;

    if (_typeof(name) === 'object') {
      return this.dispatch(_compact(_flatMap(name, function (data, n) {
        return _this.load(n, data, true);
      })));
    }

    if (!res) return;
    var coll = this[name];
    var idField = coll.idField; // normalizeLoadData

    var resPreloads = res.preloads || res.byId || (res.submits ? null : res);
    if (Array.isArray(resPreloads)) resPreloads = _mapKeys(resPreloads, function (doc, i) {
      return doc && doc[idField] || i;
    });
    var actions = []; // move tmp id to $submittedIds before loadAsMerge

    if (res.$submittedIds) {
      var $unset = Array.isArray(res.$submittedIds) ? res.$submittedIds : _keys(res.$submittedIds);
      actions.push({
        name: name,
        mutation: {
          submits: {
            $unset: $unset
          },
          originals: {
            $unset: $unset
          }
        }
      });
    }

    var mutation = {};

    if (resPreloads) {
      var now = Date.now();

      var _this$getFetchData = this.getFetchData(name),
          _byIdAts = _this$getFetchData._byIdAts;

      mutation.preloads = _mapValues(resPreloads, function (doc, id) {
        _byIdAts[id] = now;
        doc = runCast(coll, 'onLoad', doc);
        return _isPlainObject(doc) ? {
          $auto: {
            $merge: doc
          }
        } : {
          $set: doc
        };
      });
    }

    if (res.submits) mutation.submits = {
      $merge: res.submits
    };
    if (res.originals) mutation.originals = {
      $merge: res.originals
    };
    if (res.fetchAts) mutation.fetchAts = {
      $merge: res.fetchAts
    };
    actions.push({
      name: name,
      mutation: mutation
    }); // NOTE for server to pick-it back invalidate or reset data

    if (res.$invalidate) this.invalidate(name, res.$invalidate);
    if (res.$reset) this.reset(name, res.$reset);
    if (returnMutation) return actions;
    this.dispatch(actions);
  },
  // @auto-fold here
  set: function set() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var last = args.length - 1;
    args[last] = {
      $set: args[last]
    };
    this.mutate.apply(this, args);
  },
  invalidate: function invalidate(name, ids) {
    var _this2 = this;

    if (!name) return _each(this.getConfig(), function (conf, n) {
      return _this2.invalidate(n);
    });
    if (ids && ids.length === 0) return;
    this.dispatch(name, {
      _byIdAts: ids ? {
        $unset: ids
      } : {
        $set: {}
      },
      _fetchResults: {
        $set: {}
      },
      fetchAts: {
        $set: {}
      },
      preloads: ids ? {
        $unset: ids
      } : {
        $set: {}
      }
    });
  },
  reset: function reset(name, ids) {
    var _this3 = this;

    var opt = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    if (!name) return _each(this.getConfig(), function (conf, n) {
      return _this3.reset(n, null, opt);
    });
    if (ids && ids.length === 0) return;
    if (!opt.submitsOnly) this.invalidate(name, ids);
    var submits = ids ? {
      $unset: ids
    } : {
      $set: {} // use resetAt to always trigger re-render (not sure it is always needed?)

    };
    this.dispatch(name, {
      submits: submits,
      originals: submits,
      resetAt: {
        $set: Date.now()
      }
    });
  },
  // @auto-fold here
  insert: function insert(name, docs) {
    var _this4 = this;

    var inputIsArray = Array.isArray(docs);
    var inserts = inputIsArray ? docs : [docs];
    var merge = {};
    var coll = this[name];
    var idField = coll.idField;

    var insertedDocs = _map(inserts, function (doc) {
      var id = doc[idField];
      if (!id) id = doc[idField] = _this4.genId();
      doc = runCast(coll, 'onInsert', doc);
      merge[id] = doc;
      return doc;
    });

    this.mutate(name, {
      $merge: merge
    });
    return inputIsArray ? insertedDocs : insertedDocs[0];
  },
  // @auto-fold here
  update: function update(name, query, updates) {
    var oldDocs = this.pickInMemory(name, query);
    var idField = this[name].idField;
    var mut = {};

    _each(oldDocs, function (doc) {
      mut[doc[idField]] = updates;
    });

    this.mutate(name, mut);
    return oldDocs;
  },
  // @auto-fold here
  remove: function remove(name, query) {
    var removedDocs = this.pickInMemory(name, query);
    this.mutate(name, {
      $unset: _keys(removedDocs)
    });
    return removedDocs;
  }
};