import _isPlainObject from "lodash/isPlainObject";
import _pickBy from "lodash/pickBy";
import _mapValues from "lodash/mapValues";
import _each from "lodash/each";

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import collectionCore from './collectionCore';
import collectionRead from './collectionRead';
import collectionWrite from './collectionWrite';
import { mutateUtil, checkCastById } from './collection-util';

var createDb = function createDb(config) {
  var subscribers = [];

  var subscribe = function subscribe(subscriber) {
    subscribers.push(subscriber);
    var isSubscribed = true;
    return function unsubscribe() {
      if (!isSubscribed) return false;
      isSubscribed = false;
      subscribers.splice(subscribers.indexOf(subscriber), 1);
      return true;
    };
  };

  var canEmit = false;

  var db = _objectSpread({}, collectionCore, collectionRead, collectionWrite, {
    dispatch: function dispatch(actionsOrName, mut) {
      // normalize
      var actions = Array.isArray(actionsOrName) ? actionsOrName : [{
        name: actionsOrName,
        mutation: mut
      }]; // hook

      canEmit = false;
      if (db.dispatchFilter) actions = db.dispatchFilter(actions); // do change

      var change = {};
      var hasChange = false;

      _each(actions, function (_ref) {
        var name = _ref.name,
            mutation = _ref.mutation;
        var prev = db[name];
        var next = mutateUtil(prev, mutation);

        if (next !== prev) {
          next._cache = {};

          if (next.cast) {
            checkCastById('submits', next, prev, mutation);
            checkCastById('preloads', next, prev, mutation);
          }

          db[name] = change[name] = next;
          hasChange = true;
        }
      });

      canEmit = true; // assign change to db and fire to subscribers

      if (hasChange) db.emit(change);
    },
    subscribe: subscribe,
    emit: function emit(change) {
      if (canEmit) _each(subscribers, function (subscriber) {
        return subscriber(change);
      });
    },
    getConfig: function getConfig() {
      return config;
    },
    getDb: function getDb() {
      return db;
    } // create collections

  });

  var colls = _mapValues(_pickBy(config, _isPlainObject), function (conf, name) {
    return _objectSpread({
      // local persist
      submits: {},
      originals: {},
      // local memory
      _cache: {},
      // fetch persist
      preloads: {},
      fetchAts: {},
      // fetch memory
      fetchingAt: null,
      _fetchResults: {},
      _fetchPromises: {},
      _byIdAts: {},
      TYPE: 'Collection',
      idField: '_id',
      name: name,
      getDb: function getDb() {
        return db;
      }
    }, conf);
  });

  Object.assign(db, colls); // init collections

  _each(colls, function (coll, name) {
    if (coll.initState) db.load(name, coll.initState);
    coll = db[name];
    if (coll.onInit) coll.onInit(coll);
  });

  canEmit = true;
  return db;
};

export default createDb;